import { Controller, Get, Post, Body, Patch, Param, Delete, HttpCode, UseGuards, Query, ParseIntPipe, NotFoundException, Req } from '@nestjs/common'
import { TrainsService } from './trains.service'
import { CreateTrainDto } from './dto/create-train.dto'
import { UpdateTrainDto } from './dto/update-train.dto'
import { TrainEntity } from './entities/train.entity'
import { AuthenticatedRequest, JwtAuthGuard } from 'src/auth/jwt-auth.guard'
import { ApiBearerAuth, ApiCreatedResponse, ApiForbiddenResponse, ApiNotFoundResponse, ApiOkResponse, ApiUnauthorizedResponse } from '@nestjs/swagger'
import { ApiOkResponsePaginated, paginateOutput } from 'src/common/utils/pagination.utils'
import { QueryPaginationDto } from 'src/common/dto/pagination-query.dto'
import { PaginateOutputDto } from 'src/common/dto/pagination-output.dto'
import { ImportTrainDto } from './dto/import-train.dto'
import { UserFilterDto } from 'src/common/dto/user_filter.dto'

@Controller('trains')
export class TrainsController {
  constructor(private readonly trainsService: TrainsService) {}

  @Post()
  @HttpCode(201)
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiCreatedResponse({ type: TrainEntity, description: "Objet créé avec succès" })
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiForbiddenResponse({ description: "Permission refusée" })
  async create(@Body() createTrainDto: CreateTrainDto): Promise<TrainEntity> {
    const train = await this.trainsService.create(createTrainDto)
    return new TrainEntity(train)
  }

  @Get()
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiOkResponsePaginated(TrainEntity)
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiForbiddenResponse({ description: "Permission refusée" })
  async findAll(@Query() queryPagination: QueryPaginationDto, @Query() userFilter: UserFilterDto): Promise<PaginateOutputDto<TrainEntity>> {
    const [trains, total] = await this.trainsService.findAll(queryPagination, userFilter)
    return paginateOutput<TrainEntity>(trains.map(train => new TrainEntity(train)), total, queryPagination)
  }

  @Get(':id')
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiOkResponse({ type: TrainEntity })
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiForbiddenResponse({ description: "Permission refusée" })
  @ApiNotFoundResponse({ description: "Objet non trouvé" })
  async findOne(@Param('id') id: string): Promise<TrainEntity> {
    const train = await this.trainsService.findOne(id)
    if (!train)
      throw new NotFoundException(`Trajet en train inexistant avec l'identifiant ${id}`)
    return new TrainEntity(train)
  }

  @Patch(':id')
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiOkResponse({ type: TrainEntity })
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiForbiddenResponse({ description: "Permission refusée" })
  @ApiNotFoundResponse({ description: "Objet non trouvé" })
  async update(@Param('id') id: string, @Body() updateTrainDto: UpdateTrainDto) {
    return await this.trainsService.update(id, updateTrainDto)
  }

  @Delete(':id')
  @HttpCode(204)
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiOkResponse({ type: TrainEntity })
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiForbiddenResponse({ description: "Permission refusée" })
  @ApiNotFoundResponse({ description: "Objet non trouvé" })
  async remove(@Param('id') id: string) {
    await this.trainsService.remove(id)
  }

  @Post("/import")
  @HttpCode(201)
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiCreatedResponse({ type: TrainEntity, description: "Train importé avec succès" })
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiForbiddenResponse({ description: "Permission refusée" })
  async import(@Req() request: AuthenticatedRequest, @Body() importTrainDto: ImportTrainDto): Promise<TrainEntity> {
    const train = await this.trainsService.import(request.user, importTrainDto)
    return new TrainEntity(train)
  }
}
